'use strict';

exports.__esModule = true;
exports.default = undefined;

var _CellSizeAndPositionManager = require('./CellSizeAndPositionManager');

var _CellSizeAndPositionManager2 = _interopRequireDefault(_CellSizeAndPositionManager);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } } /** Virtualized list, only renders visible items.
                                                                                                                                                           *  @param  {Array<*>} data           List of data items
                                                                                                                                                           *  @param  {Function} renderRow      Renders a single row
                                                                                                                                                           *  @param  {Number}   rowHeight      Static height of a row
                                                                                                                                                           *  @param  {Number}   overscanCount  Amount of rows to render above and below visible area of the list
                                                                                                                                                           *  @param  {Function} onMount        Callback that is invoked after the list has mounted
                                                                                                                                                           *  @method {Function} scrollToIndex  Method used to scroll to any given index
                                                                                                                                                           *
                                                                                                                                                           *  @example
                                                                                                                                                           *  new VirtualizedList(element, {
                                                                                                                                                           *    data: ['a', 'b', 'c'],
                                                                                                                                                           *    renderRow: (row, index) => { <div>{row}</div> },
                                                                                                                                                           *    rowHeight: 150,
                                                                                                                                                           *    overscanCount: 5
                                                                                                                                                           *  )}
                                                                                                                                                           *
                                                                                                                                                           */

var STYLE_INNER = 'position:relative; overflow:hidden; width:100%; min-height:100%;';
var STYLE_CONTENT = 'position:absolute; top:0; left:0; height:100%; width:100%; overflow:visible;';

var VirtualizedList = function () {
  function VirtualizedList(container, options) {
    var _this = this;

    _classCallCheck(this, VirtualizedList);

    this.container = container;
    this.options = options;

    // Initialization
    this.state = {};
    this._rowSizeAndPositionManager = new _CellSizeAndPositionManager2.default({
      cellCount: options.data.length,
      cellSizeGetter: function cellSizeGetter(_ref) {
        var index = _ref.index;
        return _this.getRowHeight(index);
      },
      estimatedCellSize: options.estimatedRowHeight || 100
    });

    // Binding
    this.render = this.render.bind(this);
    this.handleScroll = this.handleScroll.bind(this);
    this.resize = this.resize.bind(this);

    // Lifecycle Methods
    this.componentWillMount();
    this.componentDidMount();
  }

  VirtualizedList.prototype.componentWillMount = function componentWillMount() {
    var inner = this.inner = document.createElement('div');
    var content = this.content = document.createElement('div');

    inner.setAttribute('style', STYLE_INNER);
    content.setAttribute('style', STYLE_CONTENT);
    inner.appendChild(content);
    this.container.appendChild(inner);
  };

  VirtualizedList.prototype.componentDidMount = function componentDidMount() {
    var onMount = this.options.onMount;

    // Resize

    this.resize(onMount);

    // Add event listeners
    this.container.addEventListener('scroll', this.handleScroll);
    window.addEventListener('resize', this.resize);
  };

  VirtualizedList.prototype.setState = function setState() {
    var _this2 = this;

    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var callback = arguments[1];

    this.state = Object.assign(this.state, state);

    window.requestAnimationFrame(function () {
      _this2.render();

      if (typeof callback === 'function') {
        callback();
      }
    });
  };

  VirtualizedList.prototype.resize = function resize(callback) {
    if (this.state.height !== this.container.offsetHeight) {
      this.setState({
        height: this.container.offsetHeight
      }, callback);
    }
  };

  VirtualizedList.prototype.handleScroll = function handleScroll() {
    this.setState({
      offset: this.container.scrollTop
    });
  };

  VirtualizedList.prototype.getRowHeight = function getRowHeight(index) {
    var rowHeight = this.options.rowHeight;


    return Array.isArray(rowHeight) ? rowHeight[index] : rowHeight;
  };

  VirtualizedList.prototype.getRowOffset = function getRowOffset(index) {
    var _rowSizeAndPositionMa = this._rowSizeAndPositionManager.getSizeAndPositionOfCell(index),
        offset = _rowSizeAndPositionMa.offset;

    return offset;
  };

  VirtualizedList.prototype.scrollToIndex = function scrollToIndex(index) {
    var offset = this.getRowOffset(index);

    this.container.scrollTop = offset;
  };

  VirtualizedList.prototype.destroy = function destroy() {
    window.removeEventListener('resize', this.resize);

    // TODO: Destroy everything properly, silly
  };

  VirtualizedList.prototype.getRowsForOffset = function getRowsForOffset(offset) {
    var _options$overscanCoun = this.options.overscanCount,
        overscanCount = _options$overscanCoun === undefined ? 3 : _options$overscanCoun;
    var height = this.state.height;

    var _rowSizeAndPositionMa2 = this._rowSizeAndPositionManager.getVisibleCellRange({
      containerSize: height,
      offset: offset
    }),
        start = _rowSizeAndPositionMa2.start,
        stop = _rowSizeAndPositionMa2.stop;

    if (overscanCount) {
      start = Math.max(0, start - start % overscanCount);
      stop += overscanCount;
    }

    return { start: start, stop: stop };
  };

  VirtualizedList.prototype.getTotalHeight = function getTotalHeight() {
    return this._rowSizeAndPositionManager.getTotalSize();
  };

  VirtualizedList.prototype.render = function render() {
    var _options = this.options,
        data = _options.data,
        renderRow = _options.renderRow;
    var _state$offset = this.state.offset,
        offset = _state$offset === undefined ? 0 : _state$offset;

    var _getRowsForOffset = this.getRowsForOffset(offset),
        start = _getRowsForOffset.start,
        stop = _getRowsForOffset.stop;

    var rows = data.slice(start, stop);
    var fragment = document.createDocumentFragment();

    rows.forEach(function (row, index) {
      return fragment.append(renderRow(row, start + index));
    });

    this.inner.style.height = this.getTotalHeight() + 'px';
    this.content.style.top = this.getRowOffset(start) + 'px';

    this.content.innerHTML = '';
    this.content.appendChild(fragment);
  };

  return VirtualizedList;
}();

exports.default = VirtualizedList;
module.exports = exports['default'];